from typing import Optional

from pydantic import Field, BaseModel


# priority: alias > origin
class Config(BaseModel):
    """
    telegram 配置类

    :配置项:

      - ``token`` / ``telegram_token``: telegram bot token
      - ``proxy`` / ``telegram_proxy``: 自定义代理
      - ``webhook_url`` / ``telegram_api_url``: WebHook 域名
      - ``polling_interval`` / ``telegram_polling_interval``: 拉取消息的间隔时间
      - ``api_server`` / ``telegram_api_server``: 自定义 API 服务器

    """

    token: str = Field(alias="telegram_token")
    proxy: Optional[str] = Field(default=None, alias="telegram_proxy")
    webhook_url: Optional[str] = Field(default=None, alias="telegram_webhook_url")
    polling_interval: float = Field(default=0.01, alias="telegram_polling_interval")
    api_server: str = Field(
        default="https://api.telegram.org/", alias="telegram_api_server"
    )

    class Config:
        extra = "ignore"
        allow_population_by_field_name = True



from typing import Any, List, Union, Optional

from nonebot.drivers import HTTPConnection

from nonebot.adapters import Bot as BaseBot

from .model import *
from .event import Event
from .message import Message, MessageSegment

class Bot(BaseBot):
    def __init__(self, self_id: str, request: HTTPConnection): ...
    async def send(
        self, event: Event, message: Union[str, Message, MessageSegment], **kwargs
    ) -> Any: ...
    async def get_updates(
        self,
        offset: Optional[int],
        limit: Optional[int],
        timeout: Optional[int],
        allowed_updates: Optional[List[str]],
    ): ...
    async def set_webhook(
        self,
        url: str,
        certificate: Optional[InputFile],
        ip_address: Optional[str],
        max_connections: Optional[int],
        allowed_updates: Optional[List[str]],
        drop_pending_updates: Optional[bool],
    ): ...
    async def delete_webhook(self, drop_pending_updates: Optional[bool]): ...
    async def get_webhook_info(self): ...
    async def get_me(self):
        """
        :说明:
          用于测试机器人 Token 的 API
        :返回:
          * ``User``: 机器人本身的 User
        """
        ...
    async def log_out(self): ...
    async def close(self): ...
    async def send_message(
        self,
        chat_id: Union[int, str],
        text: str,
        parse_mode: Optional[str],
        entities: Optional[List[MessageEntity]],
        disable_web_page_preview: Optional[bool],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def forward_message(
        self,
        chat_id: Union[int, str],
        from_chat_id: Union[int, str],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        message_id: int,
    ): ...
    async def copy_message(
        self,
        chat_id: Union[int, str],
        from_chat_id: Union[int, str],
        message_id: int,
        caption: Optional[str],
        parse_mode: Optional[str],
        caption_entities: Optional[List[MessageEntity]],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def send_photo(
        self,
        chat_id: Union[int, str],
        photo: Union[InputFile, str],
        caption: Optional[str],
        parse_mode: Optional[str],
        caption_entities: Optional[List[MessageEntity]],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def send_audio(
        self,
        chat_id: Union[int, str],
        audio: Union[InputFile, str],
        caption: Optional[str],
        parse_mode: Optional[str],
        caption_entities: Optional[List[MessageEntity]],
        duration: Optional[int],
        performer: Optional[str],
        title: Optional[str],
        thumb: Optional[Union[InputFile, str]],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def send_document(
        self,
        chat_id: Union[int, str],
        document: Union[InputFile, str],
        thumb: Optional[Union[InputFile, str]],
        caption: Optional[str],
        parse_mode: Optional[str],
        caption_entities: Optional[List[MessageEntity]],
        disable_content_type_detection: Optional[bool],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def send_video(
        self,
        chat_id: Union[int, str],
        video: Union[InputFile, str],
        duration: Optional[int],
        width: Optional[int],
        height: Optional[int],
        thumb: Optional[Union[InputFile, str]],
        caption: Optional[str],
        parse_mode: Optional[str],
        caption_entities: Optional[List[MessageEntity]],
        support_streaming: Optional[bool],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def send_animation(
        self,
        chat_id: Union[int, str],
        animation: Union[InputFile, str],
        duration: Optional[int],
        width: Optional[int],
        height: Optional[int],
        thumb: Optional[Union[InputFile, str]],
        caption: Optional[str],
        parse_mode: Optional[str],
        caption_entities: Optional[List[MessageEntity]],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def send_voice(
        self,
        chat_id: Union[int, str],
        voice: Union[InputFile, str],
        caption: Optional[str],
        parse_mode: Optional[str],
        caption_entities: Optional[List[MessageEntity]],
        duration: Optional[int],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def send_video_note(
        self,
        chat_id: Union[int, str],
        video_note: Union[InputFile, str],
        duration: Optional[int],
        length: Optional[int],
        thumb: Optional[Union[InputFile, str]],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def send_media_group(
        self,
        chat_id: Union[int, str],
        media: List[
            Union[InputMediaAudio, InputMediaDocument, InputMediaPhoto, InputMediaVideo]
        ],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
    ): ...
    async def send_location(
        self,
        chat_id: Union[int, str],
        latitude: float,
        longitude: float,
        horizontal_accuracy: Optional[float],
        live_period: Optional[int],
        heading: Optional[int],
        proximity_alert_radius: Optional[int],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def edit_message_live_location(
        self,
        chat_id: Optional[Union[int, str]],
        message_id: Optional[int],
        inline_message_id: Optional[int],
        latitude: float,
        longitude: float,
        horizontal_accuracy: Optional[float],
        heading: Optional[int],
        proximity_alert_radius: Optional[int],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def stop_message_live_location(
        self,
        chat_id: Optional[Union[int, str]],
        message_id: Optional[int],
        inline_message_id: Optional[int],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def send_venue(
        self,
        chat_id: Union[int, str],
        latitude: float,
        longitude: float,
        title: str,
        address: str,
        foursquare_id: Optional[str],
        foursquare_type: Optional[str],
        google_place_id: Optional[str],
        google_place_type: Optional[str],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def send_contact(
        self,
        chat_id: Union[int, str],
        phone_number: str,
        first_name: str,
        last_name: Optional[str],
        vcard: Optional[str],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def send_poll(
        self,
        chat_id: Union[int, str],
        question: str,
        options: List[str],
        is_anonymous: Optional[bool],
        type: Optional[str],
        allows_multiple_answers: Optional[bool],
        correct_option_id: Optional[int],
        explanation: Optional[str],
        explanation_parse_mode: Optional[str],
        explanation_entities: Optional[List[MessageEntity]],
        open_period: Optional[int],
        close_date: Optional[int],
        is_closed: Optional[bool],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def send_dice(
        self,
        chat_id: Union[int, str],
        emoji: Optional[str],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def send_chat_action(self, chat_id: Union[int, str], action: str): ...
    async def get_user_profile_photos(
        self, user_id: int, offset: Optional[int], limit: Optional[int]
    ): ...
    async def get_file(self, file_id: str): ...
    async def ban_chat_member(
        self,
        chat_id: Union[int, str],
        user_id: int,
        until_date: Optional[int],
        revoke_messages: Optional[bool],
    ): ...
    async def unban_chat_member(
        self, chat_id: Union[int, str], user_id: int, only_if_banned: Optional[bool]
    ): ...
    async def promote_chat_member(
        self,
        chat_id: Union[int, str],
        user_id: int,
        is_anonymous: Optional[bool],
        can_manage_chat: Optional[bool],
        can_post_messages: Optional[bool],
        can_edit_messages: Optional[bool],
        can_delete_messages: Optional[bool],
        can_manage_voice_chats: Optional[bool],
        can_restrict_members: Optional[bool],
        can_promote_members: Optional[bool],
        can_change_info: Optional[bool],
        can_invite_users: Optional[bool],
        can_pin_messages: Optional[bool],
    ): ...
    async def set_chat_administrator_custom_title(
        self, chat_id: Union[int, str], user_id: int, custom_title: str
    ): ...
    async def ban_chat_sender_chat(
        self, chat_id: Union[int, str], sender_chat_id: int
    ): ...
    async def unban_chat_sender_chat(
        self, chat_id: Union[int, str], sender_chat_id: int
    ): ...
    async def set_chat_permissions(
        self, chat_id: Union[int, str], permissions: ChatPermissions
    ): ...
    async def export_chat_invite_link(self, chat_id: Union[int, str]): ...
    async def create_chat_invite_link(
        self,
        chat_id: Union[int, str],
        name: Optional[str],
        expire_date: Optional[int],
        member_limit: Optional[int],
        creates_join_request: Optional[bool],
    ): ...
    async def edit_chat_invite_link(
        self,
        chat_id: Union[int, str],
        invite_link: str,
        name: Optional[str],
        expire_date: Optional[int],
        member_limit: Optional[int],
        creates_join_request: Optional[bool],
    ): ...
    async def revoke_chat_invite_link(
        self, chat_id: Union[int, str], invite_link: str
    ): ...
    async def approve_chat_join_request(
        self, chat_id: Union[int, str], user_id: int
    ): ...
    async def decline_chat_join_request(
        self, chat_id: Union[int, str], user_id: int
    ): ...
    async def set_chat_photo(self, chat_id: Union[int, str], photo: InputFile): ...
    async def delete_chat_photo(self, chat_id: Union[int, str]): ...
    async def set_chat_title(self, chat_id: Union[int, str], title: str): ...
    async def set_chat_description(
        self, chat_id: Union[int, str], description: Optional[str]
    ): ...
    async def pin_chat_message(
        self,
        chat_id: Union[int, str],
        message_id: int,
        disable_notification: Optional[bool],
    ): ...
    async def unpin_chat_message(
        self, chat_id: Union[int, str], message_id: Optional[int]
    ): ...
    async def unpin_all_chat_messages(self, chat_id: Union[int, str]): ...
    async def leave_chat(self, chat_id: Union[int, str]): ...
    async def get_chat(self, chat_id: Union[int, str]): ...
    async def get_chat_administrators(self, chat_id: Union[int, str]): ...
    async def get_chat_member_count(self, chat_id: Union[int, str]): ...
    async def get_chat_member(self, chat_id: Union[int, str], user_id: int): ...
    async def set_chat_sticker_set(
        self, chat_id: Union[int, str], sticker_set_name: str
    ): ...
    async def delete_chat_sticker_set(self, chat_id: Union[int, str]): ...
    async def answer_callback_query(
        self,
        callback_query_id: str,
        text: Optional[str],
        show_alert: Optional[bool],
        url: Optional[str],
        cache_time: Optional[int],
    ): ...
    async def set_my_commands(
        self,
        commands: List[BotCommand],
        scope: Optional[BotCommandScope],
        language_code: Optional[str],
    ): ...
    async def delete_my_commands(
        self, scope: Optional[BotCommandScope], language_code: Optional[str]
    ): ...
    async def get_my_commands(
        self, scope: Optional[BotCommandScope], language_code: Optional[str]
    ): ...
    async def edit_message_text(
        self,
        chat_id: Optional[Union[int, str]],
        message_id: Optional[int],
        inline_message_id: Optional[str],
        text: str,
        parse_mode: Optional[str],
        entities: Optional[List[MessageEntity]],
        disable_web_page_preview: Optional[bool],
        reply_markup: Optional[InlineKeyboardMarkup],
    ): ...
    async def edit_message_caption(
        self,
        chat_id: Optional[Union[int, str]],
        message_id: Optional[int],
        inline_message_id: Optional[str],
        caption: str,
        parse_mode: Optional[str],
        caption_entities: Optional[List[MessageEntity]],
        reply_markup: Optional[InlineKeyboardMarkup],
    ): ...
    async def edit_message_media(
        self,
        chat_id: Optional[Union[int, str]],
        message_id: Optional[int],
        inline_message_id: Optional[str],
        media: InputMedia,
        reply_markup: Optional[InlineKeyboardMarkup],
    ): ...
    async def edit_message_reply_markup(
        self,
        chat_id: Optional[Union[int, str]],
        message_id: Optional[int],
        inline_message_id: Optional[str],
        reply_markup: Optional[InlineKeyboardMarkup],
    ): ...
    async def stop_poll(
        self,
        chat_id: Optional[Union[int, str]],
        message_id: Optional[int],
        reply_markup: Optional[InlineKeyboardMarkup],
    ): ...
    async def delete_message(
        self, chat_id: Optional[Union[int, str]], message_id: Optional[int]
    ): ...
    async def send_sticker(
        self,
        chat_id: Union[int, str],
        sticker: Union[InputFile, str],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[
            Union[
                InlineKeyboardMarkup,
                ReplyKeyboardMarkup,
                ReplyKeyboardRemove,
                ForceReply,
            ]
        ],
    ): ...
    async def get_sticker_set(self, name: str): ...
    async def upload_sticker_file(self, user_id: int, png_sticker: InputFile): ...
    async def create_new_sticker_set(
        self,
        user_id: int,
        name: str,
        title: str,
        png_sticker: Optional[Union[InputFile, str]],
        tgs_sticker: Optional[InputFile],
        emojis: str,
        contains_masks: Optional[bool],
        mask_potion: Optional[MaskPosition],
    ): ...
    async def add_sticker_to_set(
        self,
        user_id: int,
        name: str,
        png_sticker: Optional[Union[InputFile, str]],
        tgs_sticker: Optional[InputFile],
        emojis: str,
        mask_potion: Optional[MaskPosition],
    ): ...
    async def set_sticker_position_in_set(self, sticker: str, position: int): ...
    async def delete_sticker_from_set(self, sticker: str): ...
    async def set_sticker_set_thumb(
        self, name: str, user_id: int, thumb: Optional[Union[InputFile, str]]
    ): ...
    async def answer_inline_query(
        self,
        inline_query_id: str,
        results: List[InlineQueryResult],
        cache_time: Optional[int],
        is_personal: Optional[bool],
        next_offset: Optional[str],
        switch_pm_text: Optional[str],
        switch_pm_parameter: Optional[str],
    ): ...
    async def send_invoice(
        self,
        chat_id: Union[int, str],
        title: str,
        description: str,
        payload: str,
        provider_token: str,
        prices: List[LabeledPrice],
        max_tip_amount: Optional[int],
        suggested_tip_amounts: Optional[int],
        start_parameter: Optional[str],
        provider_date: Optional[str],
        photo_url: Optional[str],
        photo_size: Optional[int],
        photo_width: Optional[int],
        photo_height: Optional[int],
        need_name: Optional[bool],
        need_phone_number: Optional[bool],
        need_email: Optional[bool],
        need_shipping_address: Optional[bool],
        send_phone_number_to_provider: Optional[bool],
        send_email_to_provider: Optional[bool],
        is_flexible: Optional[bool],
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[InlineKeyboardMarkup],
    ): ...
    async def answer_shipping_query(
        self,
        shipping_query_id: str,
        ok: bool,
        shipping_options: Optional[List[ShippingOption]],
        error_message: Optional[str],
    ): ...
    async def answer_pre_checkout_query(
        self, pre_checkout_query_id: str, ol: bool, error_message: Optional[str]
    ): ...
    async def set_passport_data_errors(
        self, user_id: int, errors: List[PassportElementError]
    ): ...
    async def send_game(
        self,
        chat_id: int,
        game_short_name: str,
        disable_notification: Optional[bool],
        protect_content: Optional[bool],
        reply_to_message_id: Optional[int],
        allow_sending_without_reply: Optional[bool],
        reply_markup: Optional[InlineKeyboardMarkup],
    ): ...
    async def set_game_score(
        self,
        user_id: int,
        score: int,
        force: Optional[bool],
        disable_edit_message: Optional[bool],
        chat_id: Optional[int],
        message_id: Optional[int],
        inline_message_id: Optional[str],
    ): ...
    async def get_game_high_score(
        self,
        user_id: int,
        chat_id: Optional[int],
        message_id: Optional[int],
        inline_message_id: Optional[str],
    ): ...



from typing import List, Union, Literal, Optional

from pydantic import Field, BaseModel


class Update(BaseModel):
    update_id: int
    message: Optional["Message_"]
    edited_message: Optional["Message_"]
    channel_post: Optional["Message_"]
    edited_channel_post: Optional["Message_"]
    inline_query: Optional["InlineQuery"]
    chosen_inline_result: Optional["ChosenInlineResult"]
    callback_query: Optional["CallbackQuery"]
    shipping_query: Optional["ShippingQuery"]
    pre_checkout_query: Optional["PreCheckoutQuery"]
    poll: Optional["Poll"]
    poll_answer: Optional["PollAnswer"]
    my_chat_member: Optional["ChatMemberUpdated"]
    chat_member: Optional["ChatMemberUpdated"]
    chat_join_request: Optional["ChatJoinRequest"]


class WebhookInfo(BaseModel):
    url: str
    has_custom_certificate: bool
    pending_update_count: int
    ip_address: Optional[str]
    last_error_date: Optional[int]
    last_error_message: Optional[str]
    max_connections: Optional[int]
    allowed_updates: Optional[List[str]]


class User(BaseModel):
    id: int
    is_bot: bool
    first_name: str
    last_name: Optional[str]
    username: Optional[str]
    language_code: Optional[str]
    """
    :说明: 用户的语言编码，比如中文用户是 zh-hans

    :类型: ``Optional[str]``
    """
    can_join_groups: Optional[bool]
    """
    :说明: 是否可加入群聊，只会在机器人的 get_me 方法中返回

    :类型: ``Optional[bool]``
    """
    can_read_all_group_messages: Optional[bool]
    """
    :说明: 是否可以读取所有群消息，只会在机器人的 get_me 方法中返回

    :类型: ``Optional[bool]``
    """
    supports_inline_queries: Optional[bool]
    """
    :说明: 是否支持 inline_queries，只会在机器人的 get_me 方法中返回

    :类型: ``Optional[bool]``
    """


class Chat(BaseModel):
    id: int
    type: Literal["private", "group", "supergroup", "channel"]
    title: Optional[str]
    """
    :说明: 聊天标题，只会在聊天类型为 group supergroups channel 时返回

    :类型: ``Optional[str]``
    """
    username: Optional[str]
    first_name: Optional[str]
    last_name: Optional[str]
    photo: Optional["ChatPhoto"]
    """
    :说明: 聊天图片，仅在使用 get_chat 方法时返回

    :类型: ``Optional[ChatPhoto]``
    """
    bio: Optional[str]
    """
    :说明: 聊天 ?，仅在使用 get_chat 方法时返回

    :类型: ``Optional[str]``
    """
    has_private_forwards: Optional[Literal[True]]
    description: Optional[str]
    """
    :说明: 聊天简介，仅在使用 get_chat 方法时返回

    :类型: ``Optional[str]``
    """
    invite_link: Optional[str]
    """
    :说明: 邀请链接，仅在使用 get_chat 方法时返回

    :类型: ``Optional[str]``
    """
    pinned_message: Optional["Message_"]
    """
    :说明: 置顶消息，仅在使用 get_chat 方法时返回

    :类型: ``Optional[Message_]``
    """
    permissions: Optional["ChatPermissions"]
    """
    :说明: 成员权限，仅在使用 get_chat 方法时返回

    :类型: ``Optional[ChatPermissions]``
    """
    slow_mode_delay: Optional[int]
    """
    :说明: 消息频率限制，仅在使用 get_chat 方法时返回

    :类型: ``Optional[int]``
    """
    message_auto_delete_time: Optional[int]
    """
    :说明: 消息自动撤回时间，仅在使用 get_chat 方法时返回

    :类型: ``Optional[int]``
    """
    has_protected_content: Optional[Literal[True]]
    sticker_set_name: Optional[str]
    """
    :说明: 聊天表情包，仅在使用 get_chat 方法时返回

    :类型: ``Optional[string]``
    """
    can_set_sticker_set: Optional[bool]
    """
    :说明: 机器人是否可设置聊天表情包，仅在使用 get_chat 方法时返回

    :类型: ``Optional[bool]``
    """
    linked_chat_id: Optional[int]
    """
    :说明: 链接到的聊天的 id，仅在使用 get_chat 方法时返回

    :类型: ``Optional[int]``
    """
    location: Optional["ChatLocation"]
    """
    :说明: 聊天地址，仅在使用 get_chat 方法时返回

    :类型: ``Optional[ChatLocation]``
    """


class Message_(BaseModel):
    message_id: int
    from_: Optional[User] = Field(default=None, alias="from")
    sender_chat: Optional[Chat]
    date: int
    chat: "Chat"
    forward_from: Optional[User]
    forward_from_chat: Optional[Chat]
    forward_from_message_id: Optional[int]
    forward_signature: Optional[str]
    forward_sender_name: Optional[str]
    forward_date: Optional[int]
    is_automatic_forward: Optional[Literal[True]]
    reply_to_message: Optional["Message_"]
    via_bot: Optional[User]
    edit_date: Optional[int]
    has_protected_content: Optional[Literal[True]]
    media_group_id: Optional[str]
    author_signature: Optional[str]
    text: Optional[str]
    entities: Optional[List["MessageEntity"]]
    animation: Optional["Animation"]
    audio: Optional["Audio"]
    document: Optional["Document"]
    photo: Optional[List["PhotoSize"]]
    sticker: Optional["Sticker"]
    video: Optional["Video"]
    video_note: Optional["VideoNote"]
    voice: Optional["Voice"]
    caption: Optional[str]
    caption_entities: Optional[List["MessageEntity"]]
    contact: Optional["Contact"]
    dice: Optional["Dice"]
    game: Optional["Game"]
    poll: Optional["Poll"]
    venue: Optional["Venue"]
    location: Optional["Location"]
    new_chat_members: Optional[List[User]]
    left_chat_member: Optional[User]
    new_chat_title: Optional[str]
    new_chat_photo: Optional[List["PhotoSize"]]
    delete_chat_photo: Optional[Literal[True]]
    group_chat_created: Optional[Literal[True]]
    supergroup_chat_created: Optional[Literal[True]]
    channel_chat_created: Optional[Literal[True]]
    message_auto_delete_timer_changed: Optional["MessageAutoDeleteTimerChanged"]
    migrate_to_chat_id: Optional[int]
    migrate_from_chat_id: Optional[int]
    pinned_message: Optional["Message_"]
    invoice: Optional["Invoice"]
    successful_payment: Optional["SuccessfulPayment"]
    connected_website: Optional[str]
    passport_data: Optional["PassportData"]
    proximity_alert_triggered: Optional["ProximityAlertTriggered"]
    voice_chat_scheduled: Optional["VoiceChatScheduled"]
    voice_chat_started: Optional["VoiceChatStarted"]
    voice_chat_ended: Optional["VoiceChatEnded"]
    voice_chat_participants_invited: Optional["VoiceChatParticipantsInvited"]
    reply_markup: Optional["InlineKeyboardMarkup"]


class MessageId(BaseModel):
    message_id: int


class MessageEntity(BaseModel):
    type: str
    offset: int
    length: int
    url: Optional[str]
    user: Optional[User]
    language: Optional[str]


class PhotoSize(BaseModel):
    file_id: str
    file_unique_id: str
    width: int
    height: int
    file_size: Optional[int]


class Animation(BaseModel):
    file_id: str
    file_unique_id: str
    width: int
    height: int
    duration: int
    thumb: Optional[PhotoSize]
    file_name: Optional[str]
    mime_type: Optional[str]
    file_size: Optional[int]


class Audio(BaseModel):
    file_id: str
    file_unique_id: str
    width: int
    height: int
    duration: int
    performer: Optional[str]
    title: Optional[str]
    file_name: Optional[str]
    mime_type: Optional[str]
    file_size: Optional[int]
    thumb: Optional[PhotoSize]


class Document(BaseModel):
    file_id: str
    file_unique_id: str
    thumb: Optional[PhotoSize]
    file_name: Optional[str]
    mime_type: Optional[str]
    file_size: Optional[int]


class Video(BaseModel):
    file_id: str
    file_unique_id: str
    width: int
    height: int
    duration: int
    thumb: Optional[PhotoSize]
    file_name: Optional[str]
    mime_type: Optional[str]
    file_size: Optional[int]


class VideoNote(BaseModel):
    file_id: str
    file_unique_id: str
    length: int
    duration: int
    thumb: Optional[PhotoSize]
    file_size: Optional[int]


class Voice(BaseModel):
    file_id: str
    file_unique_id: str
    duration: int
    mime_type: Optional[str]
    file_size: Optional[int]


class Contact(BaseModel):
    phone_number: str
    first_name: str
    last_name: Optional[str]
    user_id: Optional[int]
    vcard: Optional[str]


class Dice(BaseModel):
    emoji: str
    value: int


class PollOption(BaseModel):
    text: str
    voter_count: int


class PollAnswer(BaseModel):
    poll_id: str
    user: User
    option_ids: List[int]


class Poll(BaseModel):
    id: str
    question: str
    options: List[PollOption]
    total_voter_count: int
    is_closed: bool
    is_anonymous: bool
    type: str
    allows_multiple_answers: bool
    correct_option_id: Optional[int]
    explanation: Optional[str]
    explanation_entities: Optional[List[MessageEntity]]
    open_period: Optional[int]
    close_date: Optional[int]


class Location(BaseModel):
    longitude: float
    latitude: float
    horizontal_accuracy: Optional[float]
    live_period: Optional[int]
    heading: Optional[int]
    proximity_alert_radius: Optional[int]


class Venue(BaseModel):
    location: Location
    title: str
    address: str
    foursquare_id: Optional[str]
    foursquare_type: Optional[str]
    google_place_id: Optional[str]
    google_place_type: Optional[str]


class ProximityAlertTriggered(BaseModel):
    traveler: User
    watcher: User
    distance: int


class MessageAutoDeleteTimerChanged(BaseModel):
    message_auto_delete_time: int


class VoiceChatScheduled(BaseModel):
    start_date: int


class VoiceChatStarted(BaseModel):
    pass


class VoiceChatEnded(BaseModel):
    duration: int


class VoiceChatParticipantsInvited(BaseModel):
    users: Optional[List[User]]


class UserProfilePhotos(BaseModel):
    total_count: int
    photos: List[List[PhotoSize]]


class File(BaseModel):
    file_id: str
    file_unique_id: str
    file_size: Optional[int]
    file_path: Optional[str]


class KeyboardButtonPollType(BaseModel):
    type: Optional[str]


class KeyboardButton(BaseModel):
    text: str
    request_contact: Optional[bool]
    request_location: Optional[bool]
    request_poll: Optional[KeyboardButtonPollType]


class ReplyKeyboardMarkup(BaseModel):
    keyboard: List[List[KeyboardButton]]
    resize_keyboard: Optional[bool]
    one_time_keyboard: Optional[bool]
    input_field_placeholder: Optional[str]
    selective: Optional[bool]


class ReplyKeyboardRemove(BaseModel):
    remove_keyboard: bool = True
    selective: Optional[bool]


class LoginUrl(BaseModel):
    url: str
    forward_text: Optional[str]
    bot_username: Optional[str]
    request_write_access: Optional[bool]


class InlineKeyboardButton(BaseModel):
    text: str
    url: Optional[str]
    login_url: Optional[LoginUrl]
    callback_date: Optional[str]
    switch_inline_query: Optional[str]
    switch_inline_query_current_chat: Optional[str]
    callback_game: Optional["CallbackGame"]
    pay: Optional[bool]


class InlineKeyboardMarkup(BaseModel):
    inline_keyboard: List[List[InlineKeyboardButton]]


class CallbackQuery(BaseModel):
    id: str
    from_: Optional[User] = Field(default=None, alias="from")
    message: Optional[Message_]
    inline_message_id: Optional[str]
    chat_instance: Optional[str]
    date: Optional[str]
    game_short_name: Optional[str]


class ForceReply(BaseModel):
    force_reply: bool = True
    input_field_placeholder: Optional[str]
    selective: Optional[bool]


class ChatPhoto(BaseModel):
    small_file_id: str
    small_file_unique_id: str
    big_file_id: str
    big_file_unique_id: str


class ChatInviteLink(BaseModel):
    invite_link: str
    creator: User
    creates_join_request: bool
    is_primary: bool
    is_revoked: bool
    name: Optional[str]
    expire_date: Optional[int]
    member_limit: Optional[int]
    pending_join_request_count: Optional[int]


class ChatMember(BaseModel):
    status: str
    user: User


class ChatMemberOwner(ChatMember):
    status: str = "creator"
    is_anonymous: bool
    custom_title: str


class ChatMemberAdministrator(ChatMember):
    status: str = "administrator"
    can_be_edited: bool
    is_anonymous: bool
    can_manage_chat: bool
    can_delete_messages: bool
    can_manage_voice_chats: bool
    can_restrict_members: bool
    can_promote_members: bool
    can_change_info: bool
    can_invite_users: bool
    can_post_messages: Optional[bool]
    can_edit_messages: Optional[bool]
    can_pin_messages: Optional[bool]
    custom_title: Optional[str]


class ChatMemberMember(ChatMember):
    status: str = "member"


class ChatMemberRestricted(ChatMember):
    status: str = "restricted"
    is_member: bool
    can_change_info: bool
    can_invite_users: bool
    can_pin_messages: bool
    can_send_messages: bool
    can_send_media_messages: bool
    can_send_polls: bool
    can_send_other_messages: bool
    can_add_web_page_previews: bool
    until_date: int


class ChatMemberLeft(ChatMember):
    status: str = "left"


class ChatMemberBanned(ChatMember):
    status: str = "kicked"


class ChatMemberUpdated(BaseModel):
    chat: Chat
    from_: User = Field(alias="from")
    date: int
    old_chat_member: ChatMember
    new_chat_member: ChatMember
    invite_link: Optional[ChatInviteLink]


class ChatJoinRequest(BaseModel):
    chat: Chat
    from_: User = Field(alias="from")
    date: int
    bio: Optional[str]
    invite_link: Optional[ChatInviteLink]


class ChatPermissions(BaseModel):
    can_send_messages: Optional[bool]
    can_send_media_messages: Optional[bool]
    can_send_polls: Optional[bool]
    can_send_other_messages: Optional[bool]
    can_add_web_page_previews: Optional[bool]
    can_change_info: Optional[bool]
    can_invite_users: Optional[bool]
    can_pin_messages: Optional[bool]


class ChatLocation(BaseModel):
    location: Location
    address: str


class BotCommand(BaseModel):
    command: str
    description: str


class BotCommandScope(BaseModel):
    type: str


class BotCommandScopeDefault(BotCommandScope):
    type: str = "default"


class BotCommandScopeAllPrivateChats(BotCommandScope):
    type: str = "all_private_chats"


class BotCommandScopeAllGroupChats(BotCommandScope):
    type: str = "all_group_chats"


class BotCommandScopeAllChatAdministrators(BotCommandScope):
    type: str = "all_chat_administrators"


class BotCommandScopeChat(BotCommandScope):
    type: str = "chat"
    chat_id: Union[int, str]


class BotCommandScopeChatAdministrators(BotCommandScope):
    type: str = "chat_administrators"
    chat_id: Union[int, str]


class BotCommandScopeChatMember(BotCommandScope):
    type: str = "chat_member"
    chat_id: Union[int, str]
    user_id: int


class ResponseParameters(BaseModel):
    migrate_to_chat_id: Optional[int]
    retry_after: Optional[int]


class InputFile(BaseModel):
    pass


class InputMedia(BaseModel):
    type: str
    media: str
    caption: Optional[str]
    parse_mode: Optional[str]
    caption_entities: Optional[List[MessageEntity]]


class InputMediaPhoto(InputMedia):
    type: str = "photo"


class InputMediaVideo(InputMedia):
    type: str = "video"
    thumb: Optional[Union[InputFile, str]]
    width: Optional[int]
    height: Optional[int]
    duration: Optional[int]
    supports_streaming: Optional[bool]


class InputMediaAnimation(InputMedia):
    type: str = "animation"
    thumb: Optional[Union[InputFile, str]]
    width: Optional[int]
    height: Optional[int]
    duration: Optional[int]


class InputMediaAudio(InputMedia):
    type: str = "audio"
    thumb: Optional[Union[InputFile, str]]
    duration: Optional[int]
    performer: Optional[str]
    title: Optional[str]


class InputMediaDocument(InputMedia):
    type: str = "document"
    thumb: Optional[Union[InputFile, str]]
    disable_content_type_detection: Optional[bool]


class MaskPosition(BaseModel):
    point: str
    x_shift: float
    y_shift: float
    scale: float


class Sticker(BaseModel):
    file_id: str
    file_unique_id: str
    width: int
    height: int
    is_animated: bool
    thumb: Optional[PhotoSize]
    emoji: Optional[str]
    set_name: Optional[str]
    mask_position: Optional[MaskPosition]
    file_size: Optional[int]


class StickerSet(BaseModel):
    name: str
    title: str
    is_animated: bool
    contains_masks: bool
    stickers: List[Sticker]
    thumb: Optional[PhotoSize]


class InlineQuery(BaseModel):
    id: str
    from_: User = Field(alias="from")
    query: str
    offset: str
    chat_type: Optional[str]
    Location: Optional[Location]


class InputMessageContent(BaseModel):
    pass


class InputTextMessageContent(InputMessageContent):
    message_text: str
    parse_mode: Optional[str]
    entities: Optional[List[MessageEntity]]
    disable_web_page_preview: Optional[bool]


class InputLocationMessageContent(InputMessageContent):
    latitude: float
    longitude: float
    horizontal_accuracy: Optional[float]
    live_period: Optional[int]
    heading: Optional[int]
    proximity_alert_radius: Optional[int]


class InputVenueMessageContent(InputMessageContent):
    latitude: float
    longitude: float
    title: str
    address: str
    foursquare_id: Optional[str]
    foursquare_type: Optional[str]
    google_place_id: Optional[str]
    google_place_type: Optional[str]


class InputContactMessageContent(InputMessageContent):
    phone_number: str
    first_name: str
    last_name: Optional[str]
    vcard: Optional[str]


class InputInvoiceMessageContent(InputMessageContent):
    title: str
    description: str
    payload: str
    provider_token: str
    currency: str
    prices: List["LabeledPrice"]
    max_tip_amount: Optional[int]
    suggested_tip_amounts: Optional[List[int]]
    provider_data: Optional[str]
    photo_url: Optional[str]
    photo_size: Optional[int]
    photo_width: Optional[int]
    photo_height: Optional[int]
    need_name: Optional[bool]
    need_phone_number: Optional[bool]
    need_email: Optional[bool]
    need_shipping_address: Optional[bool]
    send_phone_number_to_provider: Optional[bool]
    send_email_to_provider: Optional[bool]
    is_flexible: Optional[bool]


class InlineQueryResult(BaseModel):
    type: str
    id: str
    reply_markup: Optional[InlineKeyboardMarkup]


class InlineQueryResultArticle(InlineQueryResult):
    type: str = "article"
    title: str
    input_message_content: InputMessageContent
    url: Optional[str]
    hide_url: Optional[bool]
    description: Optional[str]
    thumb_url: Optional[str]
    thumb_width: Optional[int]
    thumb_height: Optional[int]


class InlineQueryResultPhoto(InlineQueryResult):
    type: str = "photo"
    photo_url: str
    thumb_url: str
    photo_width: Optional[int]
    photo_height: Optional[int]
    title: Optional[str]
    description: Optional[str]
    caption: Optional[str]
    parse_mode: Optional[str]
    caption_entities: Optional[List[MessageEntity]]
    input_message_content: Optional[InputMessageContent]


class InlineQueryResultGif(InlineQueryResult):
    type: str = "gif"
    gif_url: str
    gif_width: Optional[int]
    gif_height: Optional[int]
    gif_duration: Optional[int]
    thumb_url: str
    thumb_mime_type: Optional[str]
    title: Optional[str]
    caption: Optional[str]
    parse_mode: Optional[str]
    caption_entities: Optional[List[MessageEntity]]
    input_message_content: Optional[InputMessageContent]


class InlineQueryResultMpeg4Gif(InlineQueryResult):
    type: str = "mpeg4_gif"
    mpeg4_url: str
    mpeg4_width: Optional[int]
    mpeg4_height: Optional[int]
    mpeg4_duration: Optional[int]
    thumb_url: str
    thumb_mime_type: Optional[str]
    title: Optional[str]
    caption: Optional[str]
    parse_mode: Optional[str]
    caption_entities: Optional[List[MessageEntity]]
    input_message_content: Optional[InputMessageContent]


class InlineQueryResultVideo(InlineQueryResult):
    type: str = "video"
    video_url: str
    mime_type: str
    thumb_url: str
    title: str
    caption: Optional[str]
    parse_mode: Optional[str]
    caption_entities: Optional[List[MessageEntity]]
    video_width: Optional[int]
    video_height: Optional[int]
    video_duration: Optional[int]
    description: Optional[str]
    input_message_content: Optional[InputMessageContent]


class InlineQueryResultAudio(InlineQueryResult):
    type: str = "audio"
    audio_url: str
    title: str
    caption: Optional[str]
    parse_mode: Optional[str]
    caption_entities: Optional[List[MessageEntity]]
    performer: Optional[str]
    audio_duration: Optional[int]
    input_message_content: Optional[InputMessageContent]


class InlineQueryResultVoice(InlineQueryResult):
    type: str = "voice"
    voice_url: str
    title: str
    caption: Optional[str]
    parse_mode: Optional[str]
    caption_entities: Optional[List[MessageEntity]]
    voice_duration: Optional[int]
    input_message_content: Optional[InputMessageContent]


class InlineQueryResultDocument(InlineQueryResult):
    type: str = "document"
    title: str
    caption: Optional[str]
    parse_mode: Optional[str]
    caption_entities: Optional[List[MessageEntity]]
    document_url: str
    mime_type: str
    description: Optional[str]
    input_message_content: Optional[InputMessageContent]
    thumb_url: Optional[str]
    thumb_width: Optional[int]
    thumb_height: Optional[int]


class InlineQueryResultLocation(InlineQueryResult):
    type: str = "location"
    latitude: float
    longitude: float
    title: str
    horizontal_accuracy: Optional[float]
    live_period: Optional[int]
    heading: Optional[int]
    proximity_alert_radius: Optional[int]
    input_message_content: Optional[InputMessageContent]
    thumb_url: Optional[str]
    thumb_width: Optional[int]
    thumb_height: Optional[int]


class InlineQueryResultVenue(InlineQueryResult):
    type: str = "venue"
    latitude: float
    longitude: float
    title: str
    address: str
    foursquare_id: Optional[str]
    foursquare_type: Optional[str]
    google_place_id: Optional[str]
    google_place_type: Optional[str]
    input_message_content: Optional[InputMessageContent]
    thumb_url: Optional[str]
    thumb_width: Optional[int]
    thumb_height: Optional[int]


class InlineQueryResultContact(InlineQueryResult):
    type: str = "contact"
    phone_number: str
    first_name: str
    last_name: Optional[str]
    user_id: Optional[int]
    vcard: Optional[str]
    input_message_content: Optional[InputMessageContent]
    thumb_url: Optional[str]
    thumb_width: Optional[int]
    thumb_height: Optional[int]


class InlineQueryResultGame(InlineQueryResult):
    type: str = "game"
    game_short_name: str


class InlineQueryResultCachedPhoto(InlineQueryResult):
    type: str = "photo"
    photo_file_id: str
    title: Optional[str]
    description: Optional[str]
    caption: Optional[str]
    parse_mode: Optional[str]
    caption_entities: Optional[List[MessageEntity]]
    input_message_content: Optional[InputMessageContent]


class InlineQueryResultCachedGif(InlineQueryResult):
    type: str = "gif"
    gif_file_id: str
    title: Optional[str]
    caption: Optional[str]
    parse_mode: Optional[str]
    caption_entities: Optional[List[MessageEntity]]
    input_message_content: Optional[InputMessageContent]


class InlineQueryResultCachedMpeg4Gif(InlineQueryResult):
    type: str = "mpeg4_gif"
    mpeg4_file_id: str
    title: Optional[str]
    caption: Optional[str]
    parse_mode: Optional[str]
    caption_entities: Optional[List[MessageEntity]]
    input_message_content: Optional[InputMessageContent]


class InlineQueryResultCachedSticker(InlineQueryResult):
    type: str = "sticker"
    sticker_file_id: str
    input_message_content: Optional[InputMessageContent]


class InlineQueryResultCachedDocument(InlineQueryResult):
    type: str = "document"
    title: str
    document_file_id: str
    description: Optional[str]
    caption: Optional[str]
    parse_mode: Optional[str]
    caption_entities: Optional[List[MessageEntity]]
    input_message_content: Optional[InputMessageContent]


class InlineQueryResultCachedVideo(InlineQueryResult):
    type: str = "video"
    video_file_id: str
    title: str
    description: Optional[str]
    caption: Optional[str]
    parse_mode: Optional[str]
    caption_entities: Optional[List[MessageEntity]]
    input_message_content: Optional[InputMessageContent]


class InlineQueryResultCachedVoice(InlineQueryResult):
    type: str = "voice"
    voice_file_id: str
    title: str
    caption: Optional[str]
    parse_mode: Optional[str]
    caption_entities: Optional[List[MessageEntity]]
    input_message_content: Optional[InputMessageContent]


class InlineQueryResultCachedAudio(InlineQueryResult):
    type: str = "audio"
    audio_file_id: str
    caption: Optional[str]
    parse_mode: Optional[str]
    caption_entities: Optional[List[MessageEntity]]
    input_message_content: Optional[InputMessageContent]


class ChosenInlineResult(BaseModel):
    result_id: str
    from_: User = Field(alias="from")
    Location: Optional[Location]
    inline_message_id: Optional[str]
    query: str


class LabeledPrice(BaseModel):
    label: str
    amount: int


class Invoice(BaseModel):
    title: str
    description: str
    start_parameter: str
    currency: str
    total_amount: int


class ShippingAddress(BaseModel):
    country_code: str
    state: str
    city: str
    street_line1: str
    street_line2: str
    post_code: str


class OrderInfo(BaseModel):
    name: Optional[str]
    phone_number: Optional[str]
    email: Optional[str]
    shipping_address: Optional[ShippingAddress]


class ShippingOption(BaseModel):
    id: str
    title: str
    prices: List[LabeledPrice]


class SuccessfulPayment(BaseModel):
    currency: str
    total_amount: int
    invoice_payload: str
    shipping_option_id: Optional[str]
    order_info: Optional[OrderInfo]
    telegram_payment_charge_id: str
    provider_payment_charge_id: str


class ShippingQuery(BaseModel):
    id: str
    from_: User = Field(alias="from")
    invoice_payload: str
    shipping_address: ShippingAddress


class PreCheckoutQuery(BaseModel):
    id: str
    from_: User = Field(alias="from")
    currency: str
    total_amount: int
    invoice_payload: str
    shipping_option_id: Optional[str]
    order_info: Optional[OrderInfo]


class PassportFile(BaseModel):
    file_id: str
    file_unique_id: str
    file_size: int
    file_date: int


class EncryptedPassportElement(BaseModel):
    type: str
    data: Optional[str]
    phone_number: Optional[str]
    email: Optional[str]
    files: Optional[List[PassportFile]]
    front_side: Optional[PassportFile]
    reverse_side: Optional[PassportFile]
    selfie: Optional[PassportFile]
    translation: Optional[List[PassportFile]]
    hash: str


class EncryptedCredentials(BaseModel):
    data: str
    hash: str
    secert: str


class PassportElementError(BaseModel):
    source: str
    type: str
    message: str


class PassportElementErrorDataField(PassportElementError):
    source: str = "data"
    field_name: str
    data_hash: str


class PassportElementErrorFrontSide(PassportElementError):
    source: str = "front_side"
    file_hash: str


class PassportElementErrorReverseSide(PassportElementError):
    source: str = "reverse_side"
    file_hash: str


class PassportElementErrorSelfie(PassportElementError):
    source: str = "selfie"
    file_hash: str


class PassportElementErrorFile(PassportElementError):
    source: str = "file"
    file_hash: str


class PassportElementErrorFiles(PassportElementError):
    source: str = "files"
    file_hashes: List[str]


class PassportElementErrorTranslationFile(PassportElementError):
    source: str = "translation_file"
    file_hash: str


class PassportElementErrorTranslationFiles(PassportElementError):
    source: str = "translation_files"
    file_hashes: List[str]


class PassportElementErrorUnspecified(PassportElementError):
    source: str = "unspecified"
    element_hash: str


class PassportData(BaseModel):
    data: List[EncryptedPassportElement]
    credentials: EncryptedCredentials


class Game(BaseModel):
    title: str
    description: str
    photo: List[PhotoSize]
    text: Optional[str]
    text_entities: Optional[List[MessageEntity]]
    animation: Optional[Animation]


class CallbackGame(BaseModel):
    pass


class GameHighScore(BaseModel):
    position: int
    user: User
    score: int


# 动态语言的悲哀
Update.update_forward_refs()
Chat.update_forward_refs()
Message_.update_forward_refs()
InlineKeyboardButton.update_forward_refs()



import json
import pathlib
from typing import TYPE_CHECKING, Any, Dict, Tuple, Union, Optional

import httpx
import aiofiles
from nonebot.log import logger
from nonebot.typing import overrides
from nonebot.message import handle_event
from nonebot.drivers import (
    Driver,
    HTTPResponse,
    ForwardDriver,
    ReverseDriver,
    HTTPConnection,
    HTTPPollingSetup,
)

from nonebot.adapters import Bot as BaseBot

from .event import Event, EventWithChat
from .config import Config as TelegramConfig
from .message import Message, MessageSegment

if TYPE_CHECKING:
    from nonebot.config import Config


class Bot(BaseBot):
    """
    telegram bot 适配。继承属性参考 `BaseBot <./#class-basebot>`_ 。
    """

    telegram_config: TelegramConfig
    update_offset: int = 0

    def __init__(self, self_id: str, request: HTTPConnection):
        super().__init__(self_id, request)

    @property
    @overrides(BaseBot)
    def type(self) -> str:
        """
        - 返回: ``"telegram"``
        """
        return "telegram"

    @classmethod
    def register(cls, driver: "Driver", config: "Config"):
        super().register(driver, config)
        cls.telegram_config = TelegramConfig(**config.dict())

        logger.info("Delete old webhook")
        httpx.post(
            f"{cls.telegram_config.api_server}bot{cls.telegram_config.token}/deleteWebhook"
        )

        if isinstance(driver, ForwardDriver):
            if isinstance(driver, ReverseDriver) and cls.telegram_config.webhook_url:
                logger.info("Set new webhook")
                httpx.post(
                    f"{cls.telegram_config.api_server}bot{cls.telegram_config.token}/setWebhook",
                    params={"url": f"{cls.telegram_config.webhook_url}/telegram/http"},
                )
            else:
                logger.info("Start poll")
                res = httpx.post(
                    f"{cls.telegram_config.api_server}bot{cls.telegram_config.token}/getUpdates"
                ).json()["result"]
                if res:
                    cls.update_offset = res[-1]["update_id"]
                driver.setup_http_polling(
                    HTTPPollingSetup(
                        "telegram",
                        cls.telegram_config.token.split(":", maxsplit=1)[0],
                        f"{cls.telegram_config.api_server}bot{cls.telegram_config.token}/getUpdates",
                        "post",
                        b"",
                        {},
                        "1.1",
                        cls.telegram_config.polling_interval,
                    )
                )

    @classmethod
    async def check_permission(
        cls, driver: Driver, request: HTTPConnection
    ) -> Tuple[Optional[str], Optional[HTTPResponse]]:
        """
        Telegram 的 Webhook 方式完全不带机器人本身的标识符，所以只能默认所有上报都通过
        """
        return cls.telegram_config.token.split(":", maxsplit=1)[0], HTTPResponse(200)

    async def handle_message(self, message):
        message = json.loads(message)
        if "update_id" in message:
            await self._handle_message(message)
        else:
            for msg in message["result"]:
                if msg["update_id"] > self.update_offset:
                    self.update_offset = msg["update_id"]
                    await self._handle_message(msg)

    async def _handle_message(self, message: Dict[str, Any]):
        try:
            event = Event.parse_event(message)
            await handle_event(self, event)
        except Exception as e:
            logger.opt(colors=True, exception=e).error(
                f"<r><bg #f8bbd0>Failed to handle event. Raw: {message}</bg #f8bbd0></r>"
            )

    async def _call_api(self, api: str, **data) -> Any:
        # 将方法名称改为驼峰式
        api = api.split("_", maxsplit=1)[0] + "".join(
            s.capitalize() for s in api.split("_")[1:]
        )
        files = {}
        for key, value in data.items():
            if isinstance(value, bytes):
                files[key] = value
            else:
                try:
                    async with aiofiles.open(value, "rb") as f:
                        files[key] = (pathlib.Path(value).name, await f.read())
                except:
                    pass
        for key in files:
            data.pop(key)

        async with httpx.AsyncClient(
            proxies=self.telegram_config.proxy, timeout=10
        ) as client:
            response = await client.post(
                f"{self.telegram_config.api_server}bot{self.telegram_config.token}/{api}",
                data=data,
                files=files,
            )
        return response.text

    async def send(
        self,
        event: Event,
        message: Union[str, Message, MessageSegment],
        **kwargs,
    ) -> Any:
        """
        由于 Telegram 对于不同类型的消息有不同的 API，如果需要批量发送不同类型的消息请尽量使用此方法，
        Nonebot 将会自动帮你转换成多条消息。
        """
        if isinstance(event, EventWithChat):
            if isinstance(message, str):
                return await self.send_message(chat_id=event.chat.id, text=message)
            elif isinstance(message, MessageSegment):
                if message.type == "text":
                    return await self.send_message(
                        chat_id=event.chat.id, text=message.data.get("text")
                    )
                elif message.type in [
                    "animation",
                    "audio",
                    "document",
                    "photo",
                    "video",
                    "voice",
                ]:
                    return await self.call_api(
                        f"send_{message.type}",
                        chat_id=event.chat.id,
                        **{
                            message.type: message.data.get("file"),
                            "caption": message.data.get("caption"),
                        },
                    )
            else:
                for seg in message:
                    if seg.type == "text":
                        return await self.send_message(
                            chat_id=event.chat.id, text=seg.data.get("text")
                        )
